@@ -34,8 +34,8 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.83.4.0";
-var gEnigmimeVersion = "0.83.4.0";
+var gEnigmailVersion = "0.84.0.0";
+var gEnigmimeVersion = "0.84.0.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -56,6 +56,7 @@ const ENIG_DIRSERVICE_CONTRACTID = "@mozilla.org/file/directory_service;1";
 const ENIG_MIME_CONTRACTID = "@mozilla.org/mime;1";
 const ENIG_WMEDIATOR_CONTRACTID = "@mozilla.org/rdf/datasource;1?name=window-mediator";
 const ENIG_APPSHELL_MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1"
+const ENIG_ASS_CONTRACTID = "@mozilla.org/appshell/appShellService;1";
 
 const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                               "@mozilla.org/network/file-output-stream;1";
@@ -96,6 +97,10 @@ var gUsePGPMimeOptionList = ["usePGPMimeNever", "usePGPMimePossible",
 var gEnigRecipientsSelectionOptions = ["askRecipientsNever",
                                        "askRecipientsClever",
                                        "askRecipientsAlways"];
+                                       
+var gEnigPerRecipientRules = ["perRecipientRulesNo",
+                              "perRecipientRulesManual",
+                              "perRecipientRulesAlways"];
 
 const ENIG_BUTTON_POS_0           = 1;
 const ENIG_BUTTON_POS_1           = 1 << 8;
@@ -106,7 +111,7 @@ const ENIG_THREE_BUTTON_STRINGS   = (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_P
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_1) +
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_2);
 
-
+/* moved to enigprefs.js!
 var gEnigmailPrefDefaults = {"configuredVersion":"",
                              "agentPath":"",
                              "alwaysTrustSend":true,
@@ -146,6 +151,7 @@ var gEnigmailPrefDefaults = {"configuredVersion":"",
                              "usePGPMimeOption":PGP_MIME_POSSIBLE,
                              "wrapHtmlBeforeSend":true
                             };
+*/
 
 var gEnigLogLevel = 2;     // Output only errors/warnings by default
 var gEnigDebugLog;
@@ -173,7 +179,7 @@ function EnigGetFrame(win, frameName) {
       return win.frames[j];
     }
   }
-
+               if (asd)
   return null;
 }
 
@@ -279,7 +285,6 @@ function GetEnigmailSvc() {
   return gEnigmailSvc.initialized ? gEnigmailSvc : null;
 }
 
-
 function EnigUpdate_0_80() {
   try {
     var oldVer=EnigGetPref("configuredVersion");
@@ -288,9 +293,12 @@ function EnigUpdate_0_80() {
       if (oldVer.substring(0,4)<"0.83") {
         var keySrv = EnigGetPref("keyserver");
         if (keySrv.indexOf(",") == -1) {
-          var newKeySrv = gEnigmailPrefDefaults.keyserver.replace(keySrv, "");
-          newKeySrv = newKeySrv.replace(/(^, |, $)/, "").replace(/, , /,", ");
-          EnigSetPref("keyserver", keySrv+", "+newKeySrv);
+          try {
+            var newKeySrv = EnigGetDefaultPref("keyserver").replace(keySrv, "");
+            newKeySrv = newKeySrv.replace(/(^, |, $)/, "").replace(/, , /,", ");
+            EnigSetPref("keyserver", keySrv+", "+newKeySrv);
+          }
+          catch (ex) {}
         }
       }
       if (oldVer.substring(0,4)<"0.81") {
@@ -328,7 +336,11 @@ function EnigConfigure() {
   if (buttonPressed == 1)  // Configure later
     return;
 
-  for (var prefName in gEnigmailPrefDefaults) {
+  var obj = new Object;
+  var prefList = gPrefEnigmail.getChildList("",obj);
+
+  for (var prefItem in prefList) {
+    prefName=prefList[prefItem];
     if (prefName.search(/AlertCount$/) >= 0) {
        // Reset alert count to default value
       try {
@@ -454,8 +466,12 @@ function EnigReadFileContents(localFile, maxBytes) {
   if (!localFile.exists() || !localFile.isReadable())
     throw Components.results.NS_ERROR_FAILURE;
 
+  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+  if (!ioServ)
+    throw Components.results.NS_ERROR_FAILURE;
 
-  return EnigReadURLContents("file://"+localFile.path, maxBytes);
+  var fileURI = ioServ.newFileURI(localFile);
+  return EnigReadURLContents(fileURI.asciiSpec, maxBytes);
 
 }
 
@@ -608,12 +624,12 @@ function EnigAdvPrefWindow() {
 
 function EnigHelpWindow(source) {
 
-   var helpUrl = "http://enigmail.mozdev.org/help.html";
+  input="chrome://enigmail/locale/help/"+source+".html";
 
-   if (source)
-     helpUrl += "#" + source
-
-   window.open(helpUrl);
+  EnigOpenWin("enigmail:help",
+              "chrome://enigmail/content/enigmailHelp.xul",
+              "chrome,resizable",
+              input);
 }
 
 function EnigUpgrade() {
@@ -652,23 +668,23 @@ function EnigDisplayRadioPref(prefName, prefValue, optionElementIds) {
 function EnigSetRadioPref(prefName, optionElementIds) {
   DEBUG_LOG("enigmailCommon.js: EnigSetRadioPref: "+prefName+"\n");
 
-  var groupElement = document.getElementById("enigmail_"+prefName);
-  if (groupElement) {
-    var optionElement = groupElement.selectedItem;
-
-    var prefValue = optionElement.value;
-    if (prefValue < optionElementIds.length) {
-      EnigSetPref(prefName, prefValue);
-      groupElement.value = prefValue;
+  try {
+    var groupElement = document.getElementById("enigmail_"+prefName);
+    if (groupElement) {
+      var optionElement = groupElement.selectedItem;
+      var prefValue = optionElement.value;
+      if (prefValue < optionElementIds.length) {
+        EnigSetPref(prefName, prefValue);
+        groupElement.value = prefValue;
+      }
     }
   }
+  catch (ex) {}
 }
 
 function EnigSetDefaultPrefs() {
   DEBUG_LOG("enigmailCommon.js: EnigSetDefaultPrefs\n");
-  for (var prefName in gEnigmailPrefDefaults) {
-    var prefValue = EnigGetPref(prefName);
-  }
+  // has become obsolete
 }
 
 function EnigSavePrefs() {
@@ -680,21 +696,20 @@ function EnigSavePrefs() {
 }
 
 function EnigGetPref(prefName) {
-   var defaultValue = gEnigmailPrefDefaults[prefName];
-
-   var prefValue = defaultValue;
+   var prefValue = null;
    try {
+      var prefType = gPrefEnigmail.getPrefType(prefName);
       // Get pref value
-      switch (typeof defaultValue) {
-      case "boolean":
+      switch (prefType) {
+      case gPrefEnigmail.PREF_BOOL:
          prefValue = gPrefEnigmail.getBoolPref(prefName);
          break;
 
-      case "number":
+      case gPrefEnigmail.PREF_INT:
          prefValue = gPrefEnigmail.getIntPref(prefName);
          break;
 
-      case "string":
+      case gPrefEnigmail.PREF_STRING:
          prefValue = gPrefEnigmail.getCharPref(prefName);
          break;
 
@@ -704,48 +719,62 @@ function EnigGetPref(prefName) {
      }
 
    } catch (ex) {
-      // Failed to get pref value; set pref to default value
-      switch (typeof defaultValue) {
-      case "boolean":
-         gPrefEnigmail.setBoolPref(prefName, defaultValue);
-         break;
-
-      case "number":
-         gPrefEnigmail.setIntPref(prefName, defaultValue);
-         break;
-
-      case "string":
-         gPrefEnigmail.setCharPref(prefName, defaultValue);
-         break;
-
-      default:
-         break;
-     }
+      // Failed to get pref value
+      ERROR_LOG("enigmailCommon.js: EnigGetPref: unknown prefName:"+prefName+" \n");
    }
 
-   //DEBUG_LOG("enigmailCommon.js: EnigGetPref: "+prefName+"="+prefValue+"\n");
    return prefValue;
 }
 
+function EnigGetDefaultPref(prefName) {
+  DEBUG_LOG("enigmailCommon.js: EnigGetDefaultPref: prefName="+prefName+"\n");
+  var prefValue=null;
+  try {
+    gPrefEnigmail.lockPref(prefName);
+    prefValue = EnigGetPref(prefName);
+    gPrefEnigmail.unlockPref(prefName);
+  }
+  catch (ex) {}
+
+  return prefValue;
+}
+
 function EnigSetPref(prefName, value) {
    DEBUG_LOG("enigmailCommon.js: EnigSetPref: "+prefName+", "+value+"\n");
-
-   var defaultValue = gEnigmailPrefDefaults[prefName];
-
+   var prefType;
+   try {
+     prefType = gPrefEnigmail.getPrefType(prefName);
+   }
+   catch (ex) {
+     switch (typeof value) {
+       case "boolean":
+         prefType = gPrefEnigmail.PREF_BOOL;
+         break;
+       case "integer":
+         prefType = gPrefEnigmail.PREF_INT;
+         break;
+       case "string":
+         prefType = gPrefEnigmail.PREF_STRING;
+         break;
+       default:
+         prefType = 0;
+         break;
+     }
+   }
    var retVal = false;
 
-   switch (typeof defaultValue) {
-      case "boolean":
+   switch (prefType) {
+      case gPrefEnigmail.PREF_BOOL:
          gPrefEnigmail.setBoolPref(prefName, value);
          retVal = true;
          break;
 
-      case "number":
+      case gPrefEnigmail.PREF_INT:
          gPrefEnigmail.setIntPref(prefName, value);
          retVal = true;
          break;
 
-      case "string":
+      case gPrefEnigmail.PREF_STRING:
          gPrefEnigmail.setCharPref(prefName, value);
          retVal = true;
          break;
@@ -811,7 +840,7 @@ EnigRequestObserver.prototype = {
 function EnigConvertFromUnicode(text, charset) {
   DEBUG_LOG("enigmailCommon.js: EnigConvertFromUnicode: "+charset+"\n");
 
-  if (!text || !charset || (charset.toLowerCase() == "iso-8859-1"))
+  if (!text || !charset /*|| (charset.toLowerCase() == "iso-8859-1")*/)
     return text;
 
   // Encode plaintext
@@ -830,7 +859,7 @@ function EnigConvertFromUnicode(text, charset) {
 function EnigConvertToUnicode(text, charset) {
   DEBUG_LOG("enigmailCommon.js: EnigConvertToUnicode: "+charset+"\n");
 
-  if (!text || !charset || (charset.toLowerCase() == "iso-8859-1"))
+  if (!text || !charset /*|| (charset.toLowerCase() == "iso-8859-1")*/)
     return text;
 
   // Encode plaintext
@@ -846,6 +875,30 @@ function EnigConvertToUnicode(text, charset) {
 }
 
 
+function EnigConvertGpgToUnicode(text) {
+  if (typeof(text)=="string") {
+    var a=text.search(/[\x80-\xFF]{2}/);
+    var b=0;
+    
+    while (a>=0) {
+      var ch=text.substr(a,2).toSource().substr(13,8).replace(/\\x/g, "\\u00");
+      var newCh=EnigConvertToUnicode(EnigConvertToUnicode(ch, "x-u-escaped"), "utf-8");
+      if (newCh != ch) {
+        //dump(ch+"\n");
+        var r=new RegExp(text.substr(a, 2), "g");
+        text=text.replace(r, newCh);
+      }
+      b=a+2;
+      a=text.substr(b+2).search(/[\x80-\xFF]{2}/);
+      if (a>=0) {
+        a += b+2;
+      }
+    }
+  }  
+  return text;
+} 
+
+
 function EnigGetDeepText(node, findStr) {
 
   DEBUG_LOG("enigmailCommon.js: EnigDeepText: <" + node.tagName + ">, '"+findStr+"'\n");
@@ -856,13 +909,13 @@ function EnigGetDeepText(node, findStr) {
       return "";
     }
   }
-  
+
   // EnigDumpHTML(node);
-  
+
   var plainText = EnigParseChildNodes(node);
   // Replace non-breaking spaces with plain spaces
   plainText = plainText.replace(/\xA0/g," ");
-  
+
   if (findStr) {
      if (plainText.indexOf(findStr) < 0) {
         return "";
@@ -877,16 +930,16 @@ function EnigGetDeepText(node, findStr) {
 function EnigParseChildNodes(node) {
 
   var plainText="";
-  
+
   if (node.nodeType == Node.TEXT_NODE) {
     // text node
     plainText = plainText.concat(node.data);
   }
   else {
-  
+
     if (node.nodeType == Node.ELEMENT_NODE) {
       if (node.tagName=="IMG" && node.className=="moz-txt-smily") {
-        // get the "alt" part of graphical smileys to ensure correct 
+        // get the "alt" part of graphical smileys to ensure correct
         // verification of signed messages
         if (node.getAttribute("alt")) {
             plainText = plainText.concat(node.getAttribute("alt"));
@@ -898,7 +951,7 @@ function EnigParseChildNodes(node) {
     // iterate over child nodes
     while (child) {
       if (! (child.nodeType == Node.ELEMENT_NODE &&
-            child.tagName == "BR" && 
+            child.tagName == "BR" &&
             ! child.hasChildNodes())) {
         // optimization: don't do an extra loop for the very frequent <BR> elements
         plainText = plainText.concat(EnigParseChildNodes(child));
@@ -906,7 +959,7 @@ function EnigParseChildNodes(node) {
       child = child.nextSibling;
     }
   }
-  
+
   return plainText;
 }
 
@@ -974,18 +1027,44 @@ function EnigClearPassphrase() {
   enigmailSvc.clearCachedPassphrase();
 }
 
+function EnigOpenWin (winName, spec, winOptions, optList) {
+  var windowManager = Components.classes[ENIG_APPSHELL_MEDIATOR_CONTRACTID].getService(Components.interfaces.nsIWindowMediator);
+
+  /* although accordign to the docs, this doesn't seem to work ...
+    var recentWin = windowManager.getMostRecentWindow(winName);
+  */
+  var winEnum=windowManager.getEnumerator(null);
+  var recentWin=null;
+  while (winEnum.hasMoreElements() && ! recentWin) {
+    var thisWin = winEnum.getNext();
+    if (thisWin.location.href==spec) {
+      recentWin = thisWin;
+    }
+  }
+  
+  if (recentWin) {
+    recentWin.focus();
+  } else {
+    var appShellSvc = Components.classes[ENIG_ASS_CONTRACTID].getService(Components.interfaces.nsIAppShellService);
+    var domWin = appShellSvc.hiddenDOMWindow;
+
+    domWin.openDialog(spec, winName, winOptions, optList);
+  }
+}
+
 function EnigViewAbout() {
   DEBUG_LOG("enigmailCommon.js: EnigViewAbout\n");
 
-  toOpenWindowByType("enigmail:about",
-                     "chrome://enigmail/content/enigmailAbout.xul");
+  EnigOpenWin ("about:enigmail",
+               "chrome://enigmail/content/enigmailAbout.xul",
+               "resizable,chrome");
 }
 
 function EnigViewConsole() {
   DEBUG_LOG("enigmailCommon.js: EnigViewConsole\n");
 
-  toOpenWindowByType("enigmail:console",
-                     "chrome://enigmail/content/enigmailConsole.xul");
+  EnigOpenWin("enigmail:console",
+              "chrome://enigmail/content/enigmailConsole.xul");
 }
 
 function EnigViewDebugLog() {
@@ -1154,60 +1233,70 @@ function EnigGetTempDir() {
 
 
 function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
-   DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs\n");
+  DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs\n");
 
-   for (var prefName in gEnigmailPrefDefaults) {
-      var prefElement = document.getElementById("enigmail_"+prefName);
+  var obj = new Object;
+  var prefList = gPrefEnigmail.getChildList("",obj);
 
-      if (prefElement) {
-         var defaultValue = gEnigmailPrefDefaults[prefName];
-         var prefValue = showDefault ? defaultValue : EnigGetPref(prefName);
+  for (var prefItem in prefList) {
+    var prefName=prefList[prefItem];
+    var prefElement = document.getElementById("enigmail_"+prefName);
 
-         DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs: "+prefName+"="+prefValue+"\n");
-
-         switch (typeof defaultValue) {
-         case "boolean":
-            if (showPrefs) {
-               if (prefValue) {
-                  prefElement.setAttribute("checked", "true");
-               } else {
-                  prefElement.removeAttribute("checked");
-               }
-            }
-
-            if (setPrefs) {
-               if (prefElement.checked) {
-                  EnigSetPref(prefName, true);
-               } else {
-                  EnigSetPref(prefName, false);
-               }
-            }
-
-         break;
-
-         case "number":
-            if (showPrefs)
-              prefElement.value = prefValue;
-
-            if (setPrefs) {
-               try {
-                 EnigSetPref(prefName, 0+prefElement.value);
-               } catch (ex) {
-               }
-            }
-         break;
-
-         case "string":
-            if (showPrefs)
-              prefElement.value = prefValue;
-            if (setPrefs)
-              EnigSetPref(prefName, prefElement.value);
-            break;
-
-         default:
-         }
+    if (prefElement) {
+      var prefType = gPrefEnigmail.getPrefType(prefName);
+      var prefValue;
+      if (showDefault) {
+        prefValue = EnigGetDefaultPref(prefName);
       }
-   }
+      else {
+        prefValue = EnigGetPref(prefName);
+      }
+
+      DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs: "+prefName+"="+prefValue+"\n");
+
+      switch (prefType) {
+      case gPrefEnigmail.PREF_BOOL:
+        if (showPrefs) {
+          if (prefValue) {
+            prefElement.setAttribute("checked", "true");
+          } else {
+            prefElement.removeAttribute("checked");
+          }
+        }
+
+        if (setPrefs) {
+          if (prefElement.checked) {
+            EnigSetPref(prefName, true);
+          } else {
+            EnigSetPref(prefName, false);
+          }
+        }
+
+        break;
+
+      case gPrefEnigmail.PREF_INT:
+        if (showPrefs)
+          prefElement.value = prefValue;
+
+        if (setPrefs) {
+          try {
+            EnigSetPref(prefName, 0+prefElement.value);
+          } catch (ex) {}
+        }
+        break;
+
+      case gPrefEnigmail.PREF_STRING:
+        if (showPrefs)
+          prefElement.value = prefValue;
+        if (setPrefs)
+          EnigSetPref(prefName, prefElement.value);
+        break;
+
+      default:
+        DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs: "+prefName+" does not have a type?!\n");
+      }
+    }
+  }
 }
 
 
@@ -1310,3 +1399,10 @@ function EnigFilePicker(title, displayDir, save, defaultExtension, defaultName,
 
   return file;
 }
+
+function EnigRulesEditor() {
+  EnigOpenWin("enigmail:rulesEditor",
+              "chrome://enigmail/content/enigmailRulesEditor.xul",
+              "dialog,centerscreen,resizable");
+}
+
