@@ -12,7 +12,7 @@ const EnigOutputWrap          = 32;
 const EnigOutputFormatFlowed  = 64;
 const EnigOutputCRLineBreak   = 512;
 const EnigOutputLFLineBreak   = 1024;
-
+const EnigDummyFile = "application/enigmail-dummy-file";
 
 const ENIG_ENIGMSGCOMPFIELDS_CONTRACTID = "@mozdev.org/enigmail/composefields;1";
 
@@ -172,6 +172,11 @@ function enigComposeOpen() {
       if (enigMimeService.isEncrypted(gMsgCompose.originalMsgURI)) {
         enigSetSendMode('encrypt');
       }
+      else if (typeof(gMsgCompose.compFields.draftId)=="string") {
+        if (enigMimeService.isEncrypted(gMsgCompose.compFields.draftId.replace(/\?.*$/, ""))) {
+          enigSetSendMode('encrypt');
+        }
+      }
     }
 
   }
@@ -314,23 +319,31 @@ function enigUndoEncryption( bucketList, modifiedAttachments ) {
 
   if ( modifiedAttachments && bucketList && bucketList.hasChildNodes() ) {
     // undo inline encryption of attachments
-    var node = bucketList.firstChild;
-    while (node) {
-      for (var i in modifiedAttachments) {
+    for (var i=0; i<modifiedAttachments.length; i++) {
+      var node = bucketList.firstChild;
+      while (node) {
         if (node.attachment.url == modifiedAttachments[i].newUrl) {
-          node.attachment.url = modifiedAttachments[i].origUrl;
-          node.attachment.name = modifiedAttachments[i].origName;
-          node.attachment.temporary = modifiedAttachments[i].origTemp;
-          node.attachment.contentType = modifiedAttachments[i].origCType;
-
+          if (modifiedAttachments[i].dummyFile) {
+            node.attachment=null;
+            var delNode=node;
+            node=node.nextSibling;
+            bucketList.removeChild(delNode);
+            continue;
+          }
+          else {
+            node.attachment.url = modifiedAttachments[i].origUrl;
+            node.attachment.name = modifiedAttachments[i].origName;
+            node.attachment.temporary = modifiedAttachments[i].origTemp;
+            node.attachment.contentType = modifiedAttachments[i].origCType;
+          }
           // delete encrypted file
           try {
             modifiedAttachments[i].newFile.remove(false);
           }
           catch (ex) {}
         }
+        node=node.nextSibling;
       }
-      node=node.nextSibling;
     }
 
     modifiedAttachments = null;
@@ -538,7 +551,7 @@ function enigDisplaySignClickWarn() {
   }
 }
 
-function enigConfirmBeforeSend(toAddr, sendFlags, isOffline) {
+function enigConfirmBeforeSend(toAddr, gpgKeys, sendFlags, isOffline) {
   // get confirmation before sending message
   var msgStatus = "";
 
@@ -557,8 +570,10 @@ function enigConfirmBeforeSend(toAddr, sendFlags, isOffline) {
   }
 
   var msgConfirm = (isOffline || sendFlags & nsIEnigmail.SEND_LATER)
-          ? EnigGetString("offlineSave",msgStatus,toAddr)
-          : EnigGetString("onlineSend",msgStatus,toAddr);
+          ? EnigGetString("offlineSave",msgStatus,EnigStripEmail(toAddr))
+          : EnigGetString("onlineSend",msgStatus,EnigStripEmail(toAddr));
+  if (sendFlags & ENIG_ENCRYPT)
+    msgConfirm += "\n\n"+EnigGetString("encryptKeysNote", gpgKeys);
 
   return EnigConfirm(msgConfirm);
 }
@@ -591,9 +606,16 @@ function enigSendCommand(elementId) {
 
   if (gEnigDirty) {
     // make sure the sendFlags are reset before the message is processed
-    // (it may have been set by a previously aborted send operation!)
+    // (it may have been set by a previously cancelled send operation!)
     try {
-      gMsgCompose.compFields.securityInfo.sendFlags=0;
+      if (gMsgCompose.compFields.securityInfo instanceof Components.interfaces.nsIEnigMsgCompFields) {
+        if (!gEnigSendPGPMime) {
+          gMsgCompose.compFields.securityInfo=null;
+        }
+        else {
+          gMsgCompose.compFields.securityInfo.sendFlags=0;
+        }
+      }
     }
     catch (ex){
       try {
@@ -697,7 +719,7 @@ function enigSendCommand(elementId) {
        gMsgCompose.CheckAndPopulateRecipients(true, false, dummy);
      }
      catch (ex) {}
-     
+
      var toAddrList = [];
 
      if (msgCompFields.to)  toAddrList.push(msgCompFields.to);
@@ -746,74 +768,138 @@ function enigSendCommand(elementId) {
        }
      }
 
+     var usePGPMimeOption = EnigGetPref("usePGPMimeOption");
+
+     if (gEnigSendPGPMime) {
+       // Use PGP/MIME
+       sendFlags |= nsIEnigmail.SEND_PGP_MIME;
+     }
+
      var toAddr = toAddrList.join(", ");
      var testCipher = null;
 
      var notSignedIfNotEnc= (gEnigSendModeDirty<2 && (! enigGetAccDefault("signPlain")));
 
-
      if (toAddr.length>=1) {
 
         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: toAddr="+toAddr+"\n");
-
-        if (sendFlags & ENIG_ENCRYPT) {
-          // Encrypt test message for default encryption
-          var testExitCodeObj    = new Object();
-          var testStatusFlagsObj = new Object();
-          var testErrorMsgObj    = new Object();
-
-          var testPlain = "Test Message";
-          var testUiFlags   = nsIEnigmail.UI_TEST;
-          var testSendFlags = nsIEnigmail.SEND_ENCRYPTED |
-                              nsIEnigmail.SEND_TEST |
-                              optSendFlags;
-
-          // test recipients
-          testCipher = enigmailSvc.encryptMessage(window, testUiFlags,
-                                                        testPlain,
-                                                        fromAddr, toAddr,
-                                                        testSendFlags,
-                                                        testExitCodeObj,
-                                                        testStatusFlagsObj,
-                                                        testErrorMsgObj);
-
-
-          if ((recipientsSelectionOption==2) ||
-              ((testStatusFlagsObj.value & nsIEnigmail.INVALID_RECIPIENT) &&
-               (recipientsSelectionOption>0))) {
-
-              var resultObj = new Object();
-              var inputObj = new Object();
-              inputObj.toAddr = toAddr;
-              inputObj.options = "multisel";
-              if (notSignedIfNotEnc)
-                inputObj.options += ", notsigned";
-              if (recipientsSelectionOption<2)
-                inputObj.options += ", noforcedisp";
-              inputObj.dialogHeader = EnigGetString("recipientsSelectionHdr");
-
-              window.openDialog("chrome://enigmail/content/enigmailUserSelection.xul","", "dialog,modal,centerscreen", inputObj, resultObj);
-              try {
-                if (resultObj.cancelled) return;
-                if (! resultObj.encrypt) {
-                  // encryption explicitely turned off
-                  sendFlags &= ~ENIG_ENCRYPT;
-                }
-                else {
-                  toAddr = resultObj.userList.join(", ");
-                }
-                testCipher="ok";
-                testExitCodeObj.value = 0;
-              } catch (ex) {
-                // cancel pressed -> don't send mail
-                return;
+        var perRecipientRules=EnigGetPref("perRecipientRules");
+        var repeatSelection=0;
+        while (repeatSelection<2) {
+          if (perRecipientRules>0) {
+            var matchedKeysObj  =new Object;
+            var flagsObj=new Object;
+            if (!getRecipientsKeys(toAddr, 
+                                  (repeatSelection==1),
+                                  matchedKeysObj, 
+                                  flagsObj)) {
+              return;
+            }
+           
+            if (matchedKeysObj.value) toAddr=matchedKeysObj.value;
+            if (flagsObj.value != -1) {
+              // sign:    0x0001 | 0x0002 => 3
+              // encrypt: 0x0004 | 0x0008 => 12
+              // pgpMime: 0x0010 | 0x0020 => 48
+              switch (flagsObj.value & 3) {
+               // sign
+               case 0:
+                 sendFlags &= ~ENIG_SIGN;
+                 break;
+               case 2:
+                 sendFlags |= ENIG_SIGN;
+                 break;
               }
+    
+              switch ((flagsObj.value & 12)>>2) {
+               // encrypt
+               case 0:
+                 sendFlags &= ~ENIG_ENCRYPT;
+                 break;
+               case 2:
+                 sendFlags |= ENIG_ENCRYPT;
+                 break;
+              }
+    
+              switch ((flagsObj.value & 48)>>4) {
+               // pgpMime
+               case 0:
+                 sendFlags &= ~nsIEnigmail.SEND_PGP_MIME;
+                 break;
+               case 2:
+                 sendFlags |= nsIEnigmail.SEND_PGP_MIME;
+                 break;
+              }
+            }
           }
-          if ((!testCipher || (testExitCodeObj.value != 0)) && recipientsSelectionOption==0) {
-              // Test encryption failed; turn off default encryption
-              sendFlags &= ~ENIG_ENCRYPT;
-              DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: No default encryption because test failed\n");
+          repeatSelection++;
+          
+          if (sendFlags & ENIG_ENCRYPT) {
+            // Encrypt test message for default encryption
+            var testExitCodeObj    = new Object();
+            var testStatusFlagsObj = new Object();
+            var testErrorMsgObj    = new Object();
+  
+            var testPlain = "Test Message";
+            var testUiFlags   = nsIEnigmail.UI_TEST;
+            var testSendFlags = nsIEnigmail.SEND_ENCRYPTED |
+                                nsIEnigmail.SEND_TEST |
+                                optSendFlags;
+  
+            // test recipients
+            testCipher = enigmailSvc.encryptMessage(window, testUiFlags,
+                                                          testPlain,
+                                                          fromAddr, toAddr,
+                                                          testSendFlags,
+                                                          testExitCodeObj,
+                                                          testStatusFlagsObj,
+                                                          testErrorMsgObj);
+  
+  
+            if ((recipientsSelectionOption==2) ||
+                ((testStatusFlagsObj.value & nsIEnigmail.INVALID_RECIPIENT) &&
+                 (recipientsSelectionOption>0))) {
+  
+                var resultObj = new Object();
+                var inputObj = new Object();
+                inputObj.toAddr = toAddr;
+                inputObj.options = "multisel";
+                if (perRecipientRules<2)
+                  inputObj.options += ",rulesOption"
+                if (notSignedIfNotEnc)
+                  inputObj.options += ",notsigned";
+                if (recipientsSelectionOption<2)
+                  inputObj.options += ",noforcedisp";
+                inputObj.dialogHeader = EnigGetString("recipientsSelectionHdr");
+  
+                window.openDialog("chrome://enigmail/content/enigmailUserSelection.xul","", "dialog,modal,centerscreen", inputObj, resultObj);
+                try {
+                  if (resultObj.cancelled) return;
+                  if (resultObj.perRecipientRules) {
+                    // do an extra round because the user want to set a PGP rule
+                    continue;
+                  }
+                  if (! resultObj.encrypt) {
+                    // encryption explicitely turned off
+                    sendFlags &= ~ENIG_ENCRYPT;
+                  }
+                  else {
+                    toAddr = resultObj.userList.join(", ");
+                  }
+                  testCipher="ok";
+                  testExitCodeObj.value = 0;
+                } catch (ex) {
+                  // cancel pressed -> don't send mail
+                  return;
+                }
+            }
+            if ((!testCipher || (testExitCodeObj.value != 0)) && recipientsSelectionOption==0) {
+                // Test encryption failed; turn off default encryption
+                sendFlags &= ~ENIG_ENCRYPT;
+                DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: No default encryption because test failed\n");
+            }
           }
+          repeatSelection=2;
         }
 
         if ((gotSendFlags & ENIG_ENCRYPT) &&
@@ -845,23 +931,19 @@ function enigSendCommand(elementId) {
        }
      }
 
-     var usePGPMimeOption = EnigGetPref("usePGPMimeOption");
-
-     if (gEnigSendPGPMime) {
-       // Use PGP/MIME
-       sendFlags |= nsIEnigmail.SEND_PGP_MIME;
-     }
-
      var bucketList = document.getElementById("attachmentBucket");
      var hasAttachments = bucketList && bucketList.hasChildNodes();
 
      DEBUG_LOG("enigmailMsgComposeOverlay.js: hasAttachments = "+hasAttachments+"\n");
 
-     // enable PGP/MIME if message is saved and contains attachments
-     if ( hasAttachments && (sendFlags & nsIEnigmail.SAVE_MESSAGE)) {
-        sendFlags |= nsIEnigmail.SEND_PGP_MIME;
+     if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
+       // always enable PGP/MIME if message is saved
+       sendFlags |= nsIEnigmail.SEND_PGP_MIME;
+       if (! hasAttachments) {
+         hasAttachments = enigCreateDummyAttachment();
+       }
      }
-
+     
      if ( hasAttachments &&
         (sendFlags & ENIG_ENCRYPT_OR_SIGN) &&
         !(sendFlags & nsIEnigmail.SEND_PGP_MIME) &&
@@ -923,8 +1005,7 @@ function enigSendCommand(elementId) {
      if (usingPGPMime)
        uiFlags |= nsIEnigmail.UI_PGP_MIME;
 
-     if ((sendFlags & ENIG_ENCRYPT_OR_SIGN) && (usingPGPMime ||
-          (!hasAttachments && EnigGetPref("useMimeExperimental")))) {
+     if ((sendFlags & ENIG_ENCRYPT_OR_SIGN) && usingPGPMime) {
        // Use EnigMime
        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: Using EnigMime, flags="+sendFlags+"\n");
 
@@ -1142,14 +1223,15 @@ function enigSendCommand(elementId) {
 
      if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
        goDoCommand(elementId);
-       if (! (sendFlags & nsIEnigmail.SEND_PGP_MIME))
+       //if (! (sendFlags & nsIEnigmail.SEND_PGP_MIME)) {
           enigUndoEncryption(bucketList, gEnigModifiedAttach);
+       //}
 
        return;
      }
 
      if (EnigGetPref("confirmBeforeSend")) {
-       if (!enigConfirmBeforeSend(toAddr, sendFlags, isOffline)) {
+       if (!enigConfirmBeforeSend(toAddrList.join(", "), toAddr, sendFlags, isOffline)) {
          if (gEnigProcessed)
            enigUndoEncryption(bucketList, gEnigModifiedAttach);
 
@@ -1347,14 +1429,14 @@ function enigGenericSendMessage( msgType )
 
       } else {
         // Check if the headers of composing mail can be converted to a mail charset.
-        if (msgType == nsIMsgCompDeliverMode.Now || 
+        if (msgType == nsIMsgCompDeliverMode.Now ||
           msgType == nsIMsgCompDeliverMode.Later ||
-          msgType == nsIMsgCompDeliverMode.Save || 
-          msgType == nsIMsgCompDeliverMode.SaveAsDraft || 
+          msgType == nsIMsgCompDeliverMode.Save ||
+          msgType == nsIMsgCompDeliverMode.SaveAsDraft ||
           msgType == nsIMsgCompDeliverMode.SaveAsTemplate)
         {
           var fallbackCharset = new Object;
-          if (gPromptService && 
+          if (gPromptService &&
               !gMsgCompose.checkCharsetConversion(gEnigAccountId, fallbackCharset))
           {
             var dlgTitle = sComposeMsgsBundle.getString("initErrorDlogTitle");
@@ -1593,6 +1675,8 @@ function enigEncryptAttachments(bucketList, newAttachments, window, uiFlags,
     var newUri = ioServ.newFileURI(newFile);
     fileInfo.newUrl  = newUri.asciiSpec;
     fileInfo.newFile = newFile;
+    fileInfo.dummyFile = false;
+
 
     newAttachments.push(fileInfo);
     node = node.nextSibling;
@@ -1644,6 +1728,8 @@ function enigDecryptQuote(interactive) {
   if (!enigmailSvc)
     return;
 
+  enigmailRemoveDummyAttachment();
+
   var encoderFlags = EnigOutputFormatted | EnigOutputLFLineBreak;
 
   var docText = EnigEditorGetContentsAs("text/plain", encoderFlags);
@@ -1738,6 +1824,10 @@ function enigDecryptQuote(interactive) {
 
   // Decode plaintext from charset to unicode
   plainText = EnigConvertToUnicode(plainText, charset);
+  if (EnigGetPref("keepSettingsForReply")) {
+    if (statusFlagsObj.value & nsIEnigmail.DECRYPTION_OKAY)
+      enigSetSendMode('encrypt');
+  }
 
   var exitCode = exitCodeObj.value;
 
@@ -1778,7 +1868,7 @@ function enigDecryptQuote(interactive) {
       plainText = plainText.substr(0, signOffset+1);
     }
   }
-
+  
   // Replace encrypted quote with decrypted quote
   EnigEditorSelectAll();
 
@@ -1899,7 +1989,7 @@ function EnigEditorInsertAsQuotation(plainText) {
     DEBUG_LOG("enigmailMsgComposeOverlay.js: EnigEditorInsertAsQuotation: mailEditor="+mailEditor+"\n");
 
     return mailEditor.insertAsQuotation(plainText);
-    
+
   }
 }
 
@@ -1945,7 +2035,7 @@ EnigComposeStateListener.prototype = {
 
     if (aResult== Components.results.NS_OK) {
     }
-   
+
   },
 
   SaveInFolderDone: function(folderURI) {
@@ -2019,3 +2109,48 @@ EnigDocStateListener.prototype = {
       gEnigTimeoutID = window.setTimeout(enigDecryptQuote, 10, false);
   }
 }
+  
+function enigCreateDummyAttachment() {
+  try {
+    // create dummy file
+    var fileAttach = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
+    fileAttach.initWithPath(EnigGetTempDir());
+    fileAttach.append("sig.asc")
+    fileAttach.createUnique(Components.interfaces.NORMAL_FILE_TYPE, 0600);
+    
+    // attach file 
+    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
+    ioService = ioService.getService(Components.interfaces.nsIIOService);
+    var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
+    var attachUrl = fileHandler.getURLSpecFromFile(fileAttach);
+    var attachment = Components.classes["@mozilla.org/messengercompose/attachment;1"].createInstance(Components.interfaces.nsIMsgAttachment);
+    attachment.url = attachUrl;
+    attachment.contentType=EnigDummyFile;
+    attachment.temporary = true;
+    AddAttachment(attachment);
+    
+    // make dummy file is deleted before exit
+    var extAppLauncher = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsPIExternalAppLauncher);
+    extAppLauncher.deleteTemporaryFileOnExit(fileAttach);
+    gEnigModifiedAttach = new Array();
+    gEnigModifiedAttach.push({'newUrl': attachUrl, 'dummyFile': true});
+  }
+  catch (ex) {
+    return false;
+  }
+  
+  return true;
+}
+
+function enigmailRemoveDummyAttachment() {
+  var bucketList = document.getElementById("attachmentBucket");
+  var hasAttachments = bucketList && bucketList.hasChildNodes();
+  
+  if (hasAttachments && bucketList.childNodes.length==1) {
+    if (bucketList.firstChild.attachment.contentType==EnigDummyFile) {
+      bucketList.firstChild.attachment=null;
+      bucketList.removeChild(bucketList.firstChild);
+    }
+  }
+}
+
