@@ -113,7 +113,9 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigUpdateHdrIcons: exitCode="+exitCode+", statusFlags="+statusFlags+", keyId="+keyId+", userId="+userId+", "+errorMsg+"\n");
 
   gEnigLastEncryptedURI = GetLoadedMessage();
-
+  userId=EnigConvertGpgToUnicode(userId);
+  errorMsg=EnigConvertGpgToUnicode(errorMsg);
+  
   var errorLines="";
   var fullStatusInfo=errorMsg;
   if (errorMsg)
@@ -216,7 +218,9 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
     statusInfo += "\n\n" + errorMsg;
   }
 
-  if (statusFlags & nsIEnigmail.DECRYPTION_OKAY) {
+
+  if (statusFlags & nsIEnigmail.DECRYPTION_OKAY ||
+      (gEnigStatusBar.getAttribute("encrypted")=="ok")) {
     if (!statusInfo) {
       statusInfo = EnigGetString("decryptedMsg");
     }
@@ -247,6 +251,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       }
     }
   }
+  
   gEnigSecurityInfo = { statusFlags: statusFlags,
                         keyId: keyId,
                         userId: userId,
@@ -302,8 +307,8 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       gEnigStatusBar.setAttribute("signed", "unknown");
     }
     else {
-      gEnigStatusBar.removeAttribute("signed");
-      statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
+    //  gEnigStatusBar.removeAttribute("signed");
+    //  statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
     }
 
     if (statusFlags & nsIEnigmail.DECRYPTION_OKAY) {
@@ -324,7 +329,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       gEnigStatusBar.setAttribute("encrypted", "notok");
     }
     else {
-      gEnigStatusBar.removeAttribute("encrypted");
+      // gEnigStatusBar.removeAttribute("encrypted");
     }
 
   } catch (ex) {}
@@ -432,6 +437,14 @@ function enigMsgHdrViewHide() {
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigMsgHdrViewHide\n");
   var enigmailBox = document.getElementById("expandedEnigmailBox");
   enigmailBox.collapsed=true;
+  gEnigSecurityInfo = { statusFlags: 0,
+                      keyId: "",
+                      userId: "",
+                      statusLine: "",
+                      statusInfo: "",
+                      fullStatusInfo: "" };
+
+
 }
 
 function enigMsgHdrViewUnhide() {
